/***********************************************************
*        a FREE MMC/SD-interface to SPI-slot of SAMD21     *
*                                                          *
*          by  Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      FFSPort_MMC.c                       *
*                   MMC/SD TO SPI PORT                     *
************************************************************
*      - code generation            Frank Goetze   11/2010 *
***********************************************************/

#include "../../../inc/MMC/SAMD21/FFSPort_SPI.h"

#ifndef GCLK_CLKCTRL_ID_SERCOM1_CORE
 #define GCLK_CLKCTRL_ID_SERCOM1_CORE_Val 0x15ul  /**< \brief (GCLK_CLKCTRL) SERCOM1_CORE */
 #define GCLK_CLKCTRL_ID_SERCOM1_CORE (GCLK_CLKCTRL_ID_SERCOM1_CORE_Val << GCLK_CLKCTRL_ID_Pos)
#endif

/****************** constant variables ********************/
#define CPU_CRISTAL_CLK  48000000UL                        // processor clock in Hz (48MHz needed for USB)

/******************* global variables *********************/
U08 FFSPort_MMC_CDi;                                       // remember a negative MMC-CD interrupt

/*
************************************************************
*          set max possible baudrate and return this
************************************************************
*/
U32 FFSPort_MMC_SetBR(U32 maxclk)
{
    U32 bd;
    
    bd = (CPU_CRISTAL_CLK / maxclk) + ((CPU_CRISTAL_CLK % maxclk)? 1 : 0);
    SERCOM1->SPI.CTRLA.reg &= ~SERCOM_SPI_CTRLA_ENABLE;
    SERCOM1->SPI.BAUD.reg = bd - 1;
    SERCOM1->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
    while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) __NOP();
    return(CPU_CRISTAL_CLK / bd);
}

/*
************************************************************
*            write a char and read one back
************************************************************
*/
U08 FFSPort_MMC_Send(U08 w)
{
    while(!(SERCOM1->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_DRE)) __NOP();
    SERCOM1->SPI.DATA.reg = w;
    while(!(SERCOM1->SPI.INTFLAG.reg & SERCOM_SPI_INTFLAG_RXC)) __NOP();
    return (U08)SERCOM1->SPI.DATA.reg;
}

/*
************************************************************
*              reinitialise the SPI-port
************************************************************
*/
U08 FFSPort_MMC_ReInit(void)
{
    SERCOM1->SPI.CTRLA.reg  = SERCOM_SPI_CTRLA_SWRST;
    while(SERCOM1->SPI.SYNCBUSY.bit.SWRST) __NOP();
    SERCOM1->SPI.CTRLA.reg  = SERCOM_SPI_CTRLA_DIPO(3) | SERCOM_SPI_CTRLA_DOPO(0) | SERCOM_SPI_CTRLA_CPHA | SERCOM_SPI_CTRLA_CPOL | SERCOM_SPI_CTRLA_MODE_SPI_MASTER;
    SERCOM1->SPI.BAUD.reg   = 100;
    SERCOM1->SPI.CTRLB.reg  = SERCOM_SPI_CTRLB_RXEN;
    while(SERCOM1->SPI.SYNCBUSY.bit.CTRLB) __NOP();
    SERCOM1->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
    while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) __NOP();
    FFSPort_MMC_SetBR(400000);                             // set SPI-clk to max 400kHz as startup (should never be > 400kHz, says MMC)
    return(0);                                             // return actual alltimes ok.
}

/*
************************************************************
*            card-detect inclusive a INT
************************************************************
*/
U08 FFSPort_MMC_CD(void)
{
    if(!(PORT->Group[0].IN.reg & MMC_SPI_CD) || (FFSPort_MMC_CDi)) { // card-detect
        FFSPort_MMC_CDi = 0;                               // clear INT for "was seen"
        return(1);                                         // card is/was removed
    }
    return(0);                                             // card is still detect
}

/*
************************************************************
*              initialise the SPI-port
************************************************************
*/
U08 FFSPort_MMC_Init(void)
{
    FFSPort_MMC_CDi        = 0;                            // no INT was seen
    
    PM->APBCMASK.reg      |= PM_APBCMASK_SERCOM1;
    GCLK->CLKCTRL.reg      = (GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_SERCOM1_CORE);
    SERCOM1->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_SWRST;
    while(SERCOM1->SPI.SYNCBUSY.bit.SWRST) __NOP();
    SERCOM1->SPI.CTRLA.reg = SERCOM_SPI_CTRLA_DIPO(3) | SERCOM_SPI_CTRLA_DOPO(0) | SERCOM_SPI_CTRLA_CPHA | SERCOM_SPI_CTRLA_CPOL | SERCOM_SPI_CTRLA_MODE_SPI_MASTER;
    SERCOM1->SPI.BAUD.reg  = 100;
    SERCOM1->SPI.CTRLB.reg = SERCOM_SPI_CTRLB_RXEN;
    while(SERCOM1->SPI.SYNCBUSY.bit.CTRLB) __NOP();
    PORT->Group[0].PINCFG[PIN_PA16].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA17].bit.PMUXEN = 1;
    PORT->Group[0].PINCFG[PIN_PA19].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA16/2].reg        = (PORT_PMUX_PMUXE_C | PORT_PMUX_PMUXO_C);
    PORT->Group[0].PMUX[PIN_PA18/2].reg        = (PORT_PMUX_PMUXO_C);
    SERCOM1->SPI.CTRLA.reg |= SERCOM_SPI_CTRLA_ENABLE;
    while(SERCOM1->SPI.SYNCBUSY.bit.ENABLE) __NOP();
    
    // config the additional manual line /CS  (PA18)
    PORT->Group[0].OUTSET.reg = PORT_PA18;
    PORT->Group[0].DIRSET.reg = PORT_PA18;
    
    // config the additional manual line /CD  (PA07)
    PM->APBAMASK.reg    |= PM_APBAMASK_EIC;
    GCLK->CLKCTRL.reg    = (GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_EIC);
    EIC->CTRL.bit.SWRST  = 1;
    while(EIC->STATUS.bit.SYNCBUSY) __NOP();
    PORT->Group[0].OUTSET.reg                  = PORT_PA07;
    PORT->Group[0].DIRCLR.reg                  = PORT_PA07;
    PORT->Group[0].PINCFG[PIN_PA07].reg        = (PORT_PINCFG_INEN | PORT_PINCFG_PULLEN | PORT_PINCFG_PMUXEN);
    PORT->Group[0].PMUX[PIN_PA07/2].bit.PMUXO  = MUX_PA07A_EIC_EXTINT7; 
    EIC->CONFIG[0].reg  |= (EIC_CONFIG_FILTEN7 | EIC_CONFIG_SENSE7_BOTH);
    EIC->INTENSET.reg    =  EIC_INTENSET_EXTINT7;
    NVIC_EnableIRQ(EIC_IRQn);
    EIC->CTRL.bit.ENABLE = 1;

    return(FFSPort_MMC_ReInit());
}

/***********************************************************
*       !!!!!! SPI ISR handler (card removed) !!!!!!       *
***********************************************************/
void EIC_Handler(void)
{
    if(EIC->INTFLAG.reg & EIC_INTFLAG_EXTINT7) {
        EIC->INTFLAG.reg = EIC_INTFLAG_EXTINT7;
        if(!(PORT->Group[0].IN.reg & MMC_SPI_CD))          // card-lost or card-inserted
            FFSPort_MMC_CDi = 1;                           // remember a lost-card interrupt
    }
}
